@@ -20,10 +20,15 @@ module XMonad.Layout.LayoutHints
     , layoutHintsWithPlacement
     , layoutHintsToCenter
     , LayoutHints
+    , LayoutHintsToCenter
+    , hintsEventHook
     )  where
 
 import XMonad(LayoutClass(runLayout), mkAdjust, Window,
-              Dimension, Position, Rectangle(Rectangle),D)
+              Dimension, Position, Rectangle(Rectangle), D,
+              X, refresh, Event(..), propertyNotify, wM_NORMAL_HINTS,
+              (<&&>), io, applySizeHints, whenX, isClient, withDisplay,
+              getWindowAttributes, getWMNormalHints, WindowAttributes(..))
 import qualified XMonad.StackSet as W
 
 import XMonad.Layout.Decoration(isInStack)
@@ -32,9 +37,10 @@ import XMonad.Layout.LayoutModifier(ModifiedLayout(..),
 import XMonad.Util.Types(Direction2D(..))
 import Control.Applicative((<$>))
 import Control.Arrow(Arrow((***), first, second))
-import Control.Monad(Monad(return), mapM, join)
+import Control.Monad(join)
 import Data.Function(on)
 import Data.List(sortBy)
+import Data.Monoid(All(..))
 
 import Data.Set (Set)
 import qualified Data.Set as Set
@@ -62,6 +68,14 @@ import qualified Data.Set as Set
 -- For more detailed instructions on editing the layoutHook see:
 --
 -- "XMonad.Doc.Extending#Editing_the_layout_hook"
+--
+-- To make XMonad reflect changes in window hints immediately, add
+-- 'hintsEventHook' to your 'handleEventHook'.
+--
+-- > myHandleEventHook = hintsEventHook <+> ...
+-- >
+-- > main = xmonad defaultConfig { handleEventHook = myHandleEventHook
+-- >                             , ... }
 
 layoutHints :: (LayoutClass l a) => l a -> ModifiedLayout LayoutHints l a
 layoutHints = ModifiedLayout (LayoutHints (0, 0))
@@ -235,3 +249,19 @@ centerPlacement' cf root assigned
     = (cf $ cx - cwx, cf $ cy - cwy)
     where (cx,cy) = center root
           (cwx,cwy) = center assigned
+
+-- | Event hook that refreshes the layout whenever a window changes its hints.
+hintsEventHook :: Event -> X All
+hintsEventHook (PropertyEvent { ev_event_type = t, ev_atom = a, ev_window = w })
+    | t == propertyNotify && a == wM_NORMAL_HINTS = do
+        whenX (isClient w <&&> hintsMismatch w) $ refresh
+        return (All True)
+hintsEventHook _ = return (All True)
+
+-- | True if the window's current size does not satisfy its size hints.
+hintsMismatch :: Window -> X Bool
+hintsMismatch w = withDisplay $ \d -> io $ do
+    wa <- getWindowAttributes d w
+    sh <- getWMNormalHints d w
+    let dim = (fromIntegral $ wa_width wa, fromIntegral $ wa_height wa)
+    return $ dim /= applySizeHints 0 sh dim
